/* Terraform - (C) 1997-1999 Robert Gasch (Robert_Gasch@peoplesoft.com)
 *  - www.peoplesoft.com/peoplepages/g/robert_gasch/terraform/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */


#include <string.h>
#include "TFWindow.h"
#include "HeightField.h"
#include "GlobalTrace.h"
#include "GlobalSanityCheck.h"
#include "MenuDefs.h"
#include "TFOptions.h"


// base color for each color band 
static char	*bandBaseCol[]=	
			{	
				"MidnightBlue",
				"MediumBlue",
				"Blue",
				"Dodger Blue",
				"Green2",
				"Green3",
				"Green4",
				"DarkOliveGreen4",
				"LightGoldenrod2",
				"LightGoldenrod3",
				"LightGoldenrod4",
				"LightYellow4",
				"DimGray",
				"Gray",
				"LightGray",	};

static char 	*bandBaseCol_Desert[]=
			{
				"LightGoldenrod3",
				"LightGoldenrod4",
				"DimGray",
				"LightSlateGray",
				"Gray",
				"LightGray",	};
				

guint TFWhash (gconstpointer key);			// return hash for key
gint TFWhashcmp (gconstpointer a, gconstpointer b);	// compare 

 
/*
 *  constructor: build the window with an itemfactory, buffered drawing area 
 * 	and colormaps. Store the generated menu in a hash table so we have 
 *	a handy reference later we can use for set_sensitvity calls
 */
TFWindow::TFWindow() : GuiBufferedDrawingArea (),
			d_win (GTK_WINDOW_TOPLEVEL),
       			d_vbox (FALSE, 0),
			d_accelGroup ()
{
	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "+++ TFWindow\n");

	d_itemFactory = NULL;	
	d_menuHash = NULL;
	d_linearColorMap = d_linearColorMap2 = NULL;
	d_bandColorMap = d_bandColorMap2 = NULL;

	if (TFOptions::s_fixedMenus)
		buildWindowFixedMenu ();
	else
		buildWindowFloatingMenu ();
}


/*
 *  destructor: clean up 
 */
TFWindow::~TFWindow()
{
	this->hide ();
	d_win.hide ();

	delete d_menuHash;
	delete d_linearColorMap;
	delete d_bandColorMap;
	delete [] d_mNames;

	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "--- TFWindow\n");
}


/*
 *  quit: exit
 */
void TFWindow::quit ()
{
	gtk_exit (0);
}


/*
 *  hide_impl: hide everything there is to hide
 */
//void TFWindow::hide_impl ()
//{
//	d_win.hide ();
//}


/*
 *  show_impl: show everything there is to show 
 */
//void TFWindow::show_impl ()
//{
//	this->realize ();
//	//d_win.show ();
//}


/*
 *  buildWindowFixedMenu: build a TFWindow with the menu contained in a menubar.
 */
void TFWindow::buildWindowFixedMenu ()
{
	GtkStyle		*style; 
	Gtk_Style		Style; 

	d_itemFactory = new Gtk_ItemFactory_MenuBar ("<Main>", d_accelGroup);
	d_win.add (d_vbox);
	style = Style.gtkobj();
	gdk_font_unref (style->font);
	style->font = gdk_font_load ("-*-Helvetica-Medium-R-Normal--*-100-*-*-*-*-*-*");

    	//d_menuBarHandle = static_cast<Gtk_ObjectHandle<Gtk_MenuBar>&> (d_itemFactory->get_menubar_widget(""));
	{
	// uglier than above but without compiler warning 
	Gtk_ObjectHandle<Gtk_MenuBar> a(d_itemFactory->get_menubar_widget(""));
    	d_menuBarHandle = a;
	}

	SanityCheck::bailout ((!d_menuBarHandle.get_object()), "d_menuBarHandle.get_object failed", "TFWindow::TFWindow");
	d_menuBarHandle->push_style (Style);

	addMenus ();

	d_vbox.pack_start(*d_menuBarHandle, FALSE, FALSE, 0);

	// create and attach drawing area 
	d_vbox.pack_start (*this, TRUE, TRUE, 0);

	allocColormaps ();

	// catch destroy event
	connect_to_method (destroy, this, &TFWindow::quit);

	d_win.show ();
	d_win.set_policy (FALSE, FALSE, FALSE);
	d_vbox.show();
	d_menuBarHandle->show();
	this->show();

	// since Gtk-- 0.11.0 this has to be done here in order to properly 
	// size the window when staring without loading a height field. 
	//this->set_usize (400, 400);
	this->setSize (400, 400);
	this->show ();
}


/*
 *  buildWindowFloatingMenu: build a TFWindow with a popup menu
 */
void TFWindow::buildWindowFloatingMenu ()
{
	GtkStyle		*style; 
	Gtk_Style		Style; 

	d_itemFactory = new Gtk_ItemFactory_Menu ("<Main>", d_accelGroup);
	d_win.add (d_vbox);
	style = Style.gtkobj();
	gdk_font_unref (style->font);
	style->font = gdk_font_load ("-*-Helvetica-Medium-R-Normal--*-100-*-*-*-*-*-*");

	{
	// uglier than above but without compiler warning 
	Gtk_ObjectHandle<Gtk_Menu> a(d_itemFactory->get_menu_widget(""));
    	d_menuHandle = a;
	}

	SanityCheck::bailout ((!d_menuHandle.get_object()), "d_menuHandle.get_object failed", "TFWindow::TFWindow");
	d_menuHandle->push_style (Style);

	addMenus ();

	// create and attach drawing area 
	d_vbox.pack_start (*this, TRUE, TRUE, 0);

	allocColormaps ();

	// catch destroy event
	connect_to_method (destroy, this, &TFWindow::quit);

	d_win.show ();
	d_win.set_policy (FALSE, FALSE, FALSE);
	d_vbox.show();
	this->show();

	// since Gtk-- 0.11.0 this has to be done here in order to properly 
	// size the window when staring without loading a height field. 
	//this->set_usize (400, 400);
	this->setSize (400, 400);
	this->show ();
}


/*
 *  addMenus: build up the itemFactory by adding the individual menus
 */
void TFWindow::addMenus ()
{
	char					*s, *t;
	Gtk_ObjectHandle<Gtk_CheckMenuItem>	defaultView;
	string					defaultKey (MENU_VIEW_3DWIRE);

	t = new char[80];
	d_menuHash = new Glib_Hash (TFWhash, TFWhashcmp);
	d_mNames = new char*[50];
	d_nameCount = 0;
	s = d_mNames[d_nameCount++] = strdup (MENU_FILE_BRANCH);
	d_menuHash->insert (s, d_itemFactory->create_item (s, TFWindow::buildAccel(MENU_FILE_BRANCH, ACCEL_TYPE_ALT, t), "<Branch>", 0));

	s = d_mNames[d_nameCount++] = strdup (MENU_FILE_NEW_BRANCH);
	d_menuHash->insert (s, d_itemFactory->create_item (s, 0, "<Branch>", 0));
	s = d_mNames[d_nameCount++] = strdup (MENU_FILE_NEW_SYNTHESIS);
	d_menuHash->insert (s, d_itemFactory->create_item (s, TFWindow::buildAccel(MENU_FILE_NEW_SYNTHESIS, ACCEL_TYPE_CNTRL, t), "", 
		ItemFactoryConnector<TFWindow, string>(this,&TFWindow::fileMenuCallback, s)));
	s = d_mNames[d_nameCount++] = strdup (MENU_FILE_NEW_SUBDIVIDE);
	d_menuHash->insert (s, d_itemFactory->create_item (s, TFWindow::buildAccel(MENU_FILE_NEW_SUBDIVIDE, ACCEL_TYPE_CNTRL, t), "", 
		ItemFactoryConnector<TFWindow, string>(this,&TFWindow::fileMenuCallback, s)));
	d_itemFactory->create_item ("/File/New/", 0, "<Separator>", 0);
	s = d_mNames[d_nameCount++] = strdup (MENU_FILE_NEW_RANDOM);
	d_menuHash->insert (s, d_itemFactory->create_item (s, TFWindow::buildAccel(MENU_FILE_NEW_RANDOM, ACCEL_TYPE_CNTRL, t), "", 
		ItemFactoryConnector<TFWindow, string>(this,&TFWindow::fileMenuCallback, s)));

	s = d_mNames[d_nameCount++] = strdup (MENU_FILE_OPEN);
	d_menuHash->insert (s, d_itemFactory->create_item (s, TFWindow::buildAccel(MENU_FILE_OPEN, ACCEL_TYPE_CNTRL, t), "", 
	    ItemFactoryConnector<TFWindow, string>(this,&TFWindow::fileMenuCallback, s)));
	s = d_mNames[d_nameCount++] = strdup (MENU_FILE_MERGE);
	d_menuHash->insert (s, d_itemFactory->create_item (s, TFWindow::buildAccel(MENU_FILE_MERGE, ACCEL_TYPE_CNTRL, t), "", 
	    ItemFactoryConnector<TFWindow, string>(this,&TFWindow::fileMenuCallback, s)));
	s = d_mNames[d_nameCount++] = strdup (MENU_FILE_RELOAD);
	d_menuHash->insert (s, d_itemFactory->create_item (s, TFWindow::buildAccel(MENU_FILE_RELOAD, ACCEL_TYPE_CNTRL, t), "",
	    ItemFactoryConnector<TFWindow, string>(this,&TFWindow::fileMenuCallback, s)));
	s = d_mNames[d_nameCount++] = strdup (MENU_FILE_SAVE);
	d_menuHash->insert (s, d_itemFactory->create_item (s, TFWindow::buildAccel(MENU_FILE_SAVE, ACCEL_TYPE_CNTRL, t), "",
	    ItemFactoryConnector<TFWindow, string>(this,&TFWindow::fileMenuCallback, s)));
	s = d_mNames[d_nameCount++] = strdup (MENU_FILE_SAVEAS);
	d_menuHash->insert (s, d_itemFactory->create_item (s , TFWindow::buildAccel(MENU_FILE_SAVEAS, ACCEL_TYPE_CNTRL, t), "",
	    ItemFactoryConnector<TFWindow, string>(this,&TFWindow::fileMenuCallback, s)));
	s = d_mNames[d_nameCount++] = strdup (MENU_FILE_CLOSE);
	d_menuHash->insert (s, d_itemFactory->create_item (s, TFWindow::buildAccel(MENU_FILE_CLOSE, ACCEL_TYPE_CNTRL, t), "",
	    ItemFactoryConnector<TFWindow, string>(this,&TFWindow::fileMenuCallback, s)));

	d_itemFactory->create_item ("/File/", 0, "<Separator>", 0);

	s = d_mNames[d_nameCount++] = strdup (MENU_FILE_EXPORT_BRANCH);
	d_menuHash->insert (s, d_itemFactory->create_item (s, 0, "<Branch>", 0));
	s = d_mNames[d_nameCount++] = strdup (MENU_FILE_EXPORT_POVRAY);
	d_menuHash->insert (s, d_itemFactory->create_item (s, TFWindow::buildAccel(MENU_FILE_EXPORT_POVRAY, ACCEL_TYPE_CNTRL, t), "", 
		ItemFactoryConnector<TFWindow, string>(this,&TFWindow::fileMenuCallback, s)));

	d_itemFactory->create_item ("/File/", 0, "<Separator>", 0);

	s = d_mNames[d_nameCount++] = strdup (MENU_FILE_OPTIONS);
	d_menuHash->insert (s, d_itemFactory->create_item (s, TFWindow::buildAccel(MENU_FILE_OPTIONS, ACCEL_TYPE_CNTRL, t), "", 
	    ItemFactoryConnector<TFWindow, string>(this,&TFWindow::fileMenuCallback, s)));
	s = d_mNames[d_nameCount++] = strdup (MENU_FILE_ABOUT);
	d_menuHash->insert (s, d_itemFactory->create_item (s, TFWindow::buildAccel(MENU_FILE_ABOUT, ACCEL_TYPE_CNTRL, t), "", 
	    ItemFactoryConnector<TFWindow, string>(this,&TFWindow::fileMenuCallback, s)));
	d_itemFactory->create_item ("/File/", 0, "<Separator>" ,0);
	s = d_mNames[d_nameCount++] = strdup (MENU_FILE_QUIT);
	d_menuHash->insert (s, d_itemFactory->create_item (s, TFWindow::buildAccel(MENU_FILE_QUIT, ACCEL_TYPE_CNTRL, t), "",
	    ItemFactoryConnector<TFWindow, string>(this,&TFWindow::fileMenuCallback, s)));

	s = d_mNames[d_nameCount++] = strdup (MENU_HF_BRANCH);
	d_menuHash->insert (s, d_itemFactory->create_item (s, TFWindow::buildAccel(MENU_HF_BRANCH, ACCEL_TYPE_ALT, t), "<Branch>", 0));

	s = d_mNames[d_nameCount++] = strdup (MENU_HF_CRATERS);
	d_menuHash->insert (s, d_itemFactory->create_item (s, TFWindow::buildAccel(MENU_HF_CRATERS, ACCEL_TYPE_CNTRL, t), "", 
	    ItemFactoryConnector<TFWindow, string>(this,&TFWindow::hfMenuCallback, s)));
	s = d_mNames[d_nameCount++] = strdup (MENU_HF_FILL);
	d_menuHash->insert (s, d_itemFactory->create_item (s, TFWindow::buildAccel(MENU_HF_FILL, ACCEL_TYPE_CNTRL, t), "", 
	    ItemFactoryConnector<TFWindow, string>(this,&TFWindow::hfMenuCallback, s)));
	s = d_mNames[d_nameCount++] = strdup (MENU_HF_FOLD);
	d_menuHash->insert (s, d_itemFactory->create_item (s, TFWindow::buildAccel(MENU_HF_FOLD, ACCEL_TYPE_CNTRL, t), "",
	    ItemFactoryConnector<TFWindow, string>(this,&TFWindow::hfMenuCallback, s)));
	s = d_mNames[d_nameCount++] = strdup (MENU_HF_GAUSS_HILL);
	d_menuHash->insert (s, d_itemFactory->create_item (s, TFWindow::buildAccel(MENU_HF_GAUSS_HILL, ACCEL_TYPE_CNTRL, t), "",
	    ItemFactoryConnector<TFWindow, string>(this,&TFWindow::hfMenuCallback, s)));
	s = d_mNames[d_nameCount++] = strdup (MENU_HF_LINSCALE);
	d_menuHash->insert (s, d_itemFactory->create_item (s, TFWindow::buildAccel(MENU_HF_LINSCALE, ACCEL_TYPE_CNTRL, t), "",
	    ItemFactoryConnector<TFWindow, string>(this,&TFWindow::hfMenuCallback, s)));
	s = d_mNames[d_nameCount++] = strdup (MENU_HF_MIRROR);
	d_menuHash->insert (s, d_itemFactory->create_item (s, TFWindow::buildAccel(MENU_HF_MIRROR, ACCEL_TYPE_CNTRL, t), "",
	    ItemFactoryConnector<TFWindow, string>(this,&TFWindow::hfMenuCallback, s)));
	s = d_mNames[d_nameCount++] = strdup (MENU_HF_ROTATE);
	d_menuHash->insert (s, d_itemFactory->create_item (s, TFWindow::buildAccel(MENU_HF_ROTATE, ACCEL_TYPE_CNTRL, t), "",
	    ItemFactoryConnector<TFWindow, string>(this,&TFWindow::hfMenuCallback, s)));
	s = d_mNames[d_nameCount++] = strdup (MENU_HF_ROUGHSMOOTH);
	d_menuHash->insert (s, d_itemFactory->create_item (s, TFWindow::buildAccel(MENU_HF_ROUGHSMOOTH, ACCEL_TYPE_CNTRL, t), "",
	    ItemFactoryConnector<TFWindow, string>(this,&TFWindow::hfMenuCallback, s)));
	s = d_mNames[d_nameCount++] = strdup (MENU_HF_TERRACE);
	d_menuHash->insert (s, d_itemFactory->create_item (s, TFWindow::buildAccel(MENU_HF_TRANSFORM, ACCEL_TYPE_CNTRL, t), "",
	    ItemFactoryConnector<TFWindow, string>(this,&TFWindow::hfMenuCallback, s)));
	s = d_mNames[d_nameCount++] = strdup (MENU_HF_TRANSFORM);
	d_menuHash->insert (s, d_itemFactory->create_item (s, TFWindow::buildAccel(MENU_HF_TRANSFORM, ACCEL_TYPE_CNTRL, t), "",
	    ItemFactoryConnector<TFWindow, string>(this,&TFWindow::hfMenuCallback, s)));

	d_itemFactory->create_item ("/HF/", 0, "<Separator>", 0);

	s = d_mNames[d_nameCount++] = strdup (MENU_HF_CLONE);
	d_menuHash->insert (s, d_itemFactory->create_item (s, TFWindow::buildAccel(MENU_HF_CLONE, ACCEL_TYPE_CNTRL, t), "", 
	    ItemFactoryConnector<TFWindow, string>(this,&TFWindow::hfMenuCallback, s)));
	s = d_mNames[d_nameCount++] = strdup (MENU_HF_EXPORT_CONTOUR);
	d_menuHash->insert (s, d_itemFactory->create_item (s, TFWindow::buildAccel(MENU_HF_EXPORT_CONTOUR, ACCEL_TYPE_CNTRL, t), "", 
	    ItemFactoryConnector<TFWindow, string>(this,&TFWindow::hfMenuCallback, s)));
	s = d_mNames[d_nameCount++] = strdup (MENU_HF_ERODE);
	d_menuHash->insert (s, d_itemFactory->create_item (s, TFWindow::buildAccel(MENU_HF_ERODE, ACCEL_TYPE_CNTRL, t), "",
	    ItemFactoryConnector<TFWindow, string>(this,&TFWindow::hfMenuCallback, s)));
	s = d_mNames[d_nameCount++] = strdup (MENU_HF_FLOWMAP);
	d_menuHash->insert (s, d_itemFactory->create_item (s, TFWindow::buildAccel(MENU_HF_FLOWMAP, ACCEL_TYPE_CNTRL, t), "", 
	    ItemFactoryConnector<TFWindow, string>(this,&TFWindow::hfMenuCallback, s)));
	s = d_mNames[d_nameCount++] = strdup (MENU_HF_INVERT);
	d_menuHash->insert (s, d_itemFactory->create_item (s, TFWindow::buildAccel(MENU_HF_INVERT, ACCEL_TYPE_CNTRL, t), "",
	    ItemFactoryConnector<TFWindow, string>(this,&TFWindow::hfMenuCallback, s)));
	s = d_mNames[d_nameCount++] = strdup (MENU_HF_PLACEPOV);
	d_menuHash->insert (s, d_itemFactory->create_item (s, TFWindow::buildAccel(MENU_HF_PLACEPOV, ACCEL_TYPE_CNTRL, t), "",
	    ItemFactoryConnector<TFWindow, string>(this,&TFWindow::hfMenuCallback, s)));
	s = d_mNames[d_nameCount++] = strdup (MENU_HF_RESCALE);
	d_menuHash->insert (s, d_itemFactory->create_item (s, TFWindow::buildAccel(MENU_HF_RESCALE, ACCEL_TYPE_CNTRL, t), "",
	    ItemFactoryConnector<TFWindow, string>(this,&TFWindow::hfMenuCallback, s)));

	s = d_mNames[d_nameCount++] = strdup (MENU_VIEW_BRANCH);
	d_menuHash->insert (s, d_itemFactory->create_item (s, TFWindow::buildAccel(MENU_VIEW_BRANCH, ACCEL_TYPE_ALT, t), "<Branch>", 0));

	s = d_mNames[d_nameCount++] = strdup (MENU_VIEW_2DPLANE);
	d_menuHash->insert (s, d_itemFactory->create_item (s, "<control>1", "<RadioItem>",
	    ItemFactoryConnector<TFWindow, string>(this,&TFWindow::viewMenuCallback, s)));
	s = d_mNames[d_nameCount++] = strdup (MENU_VIEW_3DWIRE);
	d_menuHash->insert (s, d_itemFactory->create_item (s, "<control>2", MENU_VIEW_2DPLANE,
	    ItemFactoryConnector<TFWindow, string>(this,&TFWindow::viewMenuCallback, s)));
	s = d_mNames[d_nameCount++] = strdup (MENU_VIEW_3DHEIGHT);
	d_menuHash->insert (s, d_itemFactory->create_item (s, "<control>3", MENU_VIEW_2DPLANE,
	    ItemFactoryConnector<TFWindow, string>(this,&TFWindow::viewMenuCallback, s)));
	s = d_mNames[d_nameCount++] = strdup (MENU_VIEW_3DLIGHT);
	d_menuHash->insert (s, d_itemFactory->create_item (s, "<control>4", MENU_VIEW_2DPLANE,
	    ItemFactoryConnector<TFWindow, string>(this,&TFWindow::viewMenuCallback, s)));

	d_itemFactory->create_item ("/View/", 0, "<Separator>", 0);

	s = d_mNames[d_nameCount++] = strdup (MENU_VIEW_FASTWIRE);
	d_menuHash->insert (s, d_itemFactory->create_item (s, "<control>5", "<ToggleItem>",
	    ItemFactoryConnector<TFWindow, string>(this,&TFWindow::viewMenuCallback, s)));
	s = d_mNames[d_nameCount++] = strdup (MENU_VIEW_AUTOROTATE);
	d_menuHash->insert (s, d_itemFactory->create_item (s, "<control>6", "<ToggleItem>",
	    ItemFactoryConnector<TFWindow, string>(this,&TFWindow::viewMenuCallback, s)));

	s = d_mNames[d_nameCount++] = strdup (MENU_CMAP_BRANCH);
	d_menuHash->insert (s, d_itemFactory->create_item (s, TFWindow::buildAccel(MENU_VIEW_BRANCH, ACCEL_TYPE_ALT, t), "<Branch>", 0));

	s = d_mNames[d_nameCount++] = strdup (MENU_CMAP_COLORBANDS);
	d_menuHash->insert (s, d_itemFactory->create_item (s, "<alt>1", "<RadioItem>", 
	    ItemFactoryConnector<TFWindow, string>(this,&TFWindow::colormapMenuCallback, s)));
	s = d_mNames[d_nameCount++] = strdup (MENU_CMAP_DESERT);
	d_menuHash->insert (s, d_itemFactory->create_item (s, "<alt>2", MENU_CMAP_COLORBANDS, 
	    ItemFactoryConnector<TFWindow, string>(this,&TFWindow::colormapMenuCallback, s)));
	s = d_mNames[d_nameCount++] = strdup (MENU_CMAP_GRAYSCALE);
	d_menuHash->insert (s, d_itemFactory->create_item (s, "<alt>3", MENU_CMAP_COLORBANDS,
	    ItemFactoryConnector<TFWindow, string>(this,&TFWindow::colormapMenuCallback, s)));
	s = d_mNames[d_nameCount++] = strdup (MENU_CMAP_REDHOT);
	d_menuHash->insert (s, d_itemFactory->create_item (s, "<alt>4", MENU_CMAP_COLORBANDS,
	    ItemFactoryConnector<TFWindow, string>(this,&TFWindow::colormapMenuCallback, s)));

	d_win.add_accel_group (d_accelGroup);

	defaultView = d_itemFactory->get_checkmenuitem_widget (defaultKey);
	defaultView->set_active (TRUE);
	delete [] t;
}


/*
 *  allocColormaps: allocate the colormaps 
 */
void TFWindow::allocColormaps ()
{
	// create colormaps
	d_linearColorMap = new GuiColormapLinear (this->get_toplevel(), 255);
	d_linearColorMap->allocColors (225, 0, 0, 0, 1, 1, 1);

	d_linearColorMap2 = new GuiColormapLinear (this->get_toplevel(), 255);
	d_linearColorMap2->allocColors (225, 0, 1, 0, 1, 0, 0);

	d_bandColorMap = new GuiColormapBands (this->get_toplevel(), 15, 15, 4);
	d_bandColorMap->allocColorBands (bandBaseCol); 

	d_bandColorMap2 = new GuiColormapBands (this->get_toplevel(), 6, 15, 1);
	d_bandColorMap2->allocColorBands (bandBaseCol_Desert); 

	d_useLinearColormap = FALSE;
}


/*
 *  fileMenuCallback: default file menu handler
 */
void TFWindow::fileMenuCallback(string m)
{
	if (m == "Exit")
		quit ();
	//cout << "Callback " << m << " caught " << endl;
}


/*
 *  hfMenuCallback: default HF menu handler
 */
void TFWindow::hfMenuCallback(string m)
{
	//cout << "Callback " << m << " caught " << endl;
}


/*
 *  viewMenuCallback: default view menu handler
 */
void TFWindow::viewMenuCallback(string m)
{
	//cout << "Callback " << m << " caught " << endl;
}


/*
 *  colormapMenuCallback: default colormap menu handler
 */
void TFWindow::colormapMenuCallback(string m)
{
	//cout << "Callback " << m << " caught " << endl;
}



/*
 *  setSize: set the size of all the relevant widgets depending on the 
 * 	menu mode used. 
 */
void TFWindow::setSize (int x, int y)
{
	if ((this->width() != x) || (this->height() != y))
		{
		if (TFOptions::s_fixedMenus)
			d_win.set_usize (x, y+d_menuBarHandle->height());
		else
			d_win.set_usize (x, y);
		this->setUsize (x, y);

		d_win.show ();
		this->show();
		gdk_flush ();

		Gtk_Main *gtkMain = Gtk_Main::instance();
		while (gtkMain->events_pending())
			gtkMain->iteration ();
		}
}


/*
 *  buildAccel: the a menu name and extract the accerlator key by 
 * 	searching for the underscore (_) marker. 
 * 
 * 	accelType denotes the mod key used: 
 *		'a'		ALT
 *		'c'		CNTRL
 * 	MenuDefs.h defines these as ACCEL_TYPE_*
 */
char *TFWindow::buildAccel (char *menu, char accelType, char *s)
{
	SanityCheck::bailout ((!menu), "menu==NULL", "TFWindow::buildAccel");
	SanityCheck::bailout ((!s), "s==NULL", "TFWindow::buildAccel");

	if (!accelType)
		return NULL;

	char 	*accelKey = menu;
	char	*accel = s;

	while (*accelKey)
		{
		if (*accelKey == '_')
			{
			accelKey++;		// advance to accelrator key
			break;
			}
		accelKey++;
		}

	if (! (*accelKey))			// no accelerator found
		return NULL;

	strcpy (accel, "");
	if (accelType == 'a')
		sprintf (accel, "<alt>%c", toupper(*accelKey));
	else
	if (accelType == 'c')
		sprintf (accel, "<control>%c", toupper(*accelKey));
	else
		SanityCheck::bailout ((TRUE), "accelType out of Range", "TFWindow::buildAccel");

	// printf ("Menu = %s,   Accel = %s\n", menu, accel);
	if (strlen(accel))
		return accel;
	else	
		return NULL;
}


guint TFWhash (gconstpointer key)
{
	char 	*s = (char *)key;
	int	i, v=0,
		l=strlen(s);

	for (i=0; i<l; i++)
		v = (64*v + s[i])%101;

	return (v);
}


gint TFWhashcmp (gconstpointer a, gconstpointer b)
{
	return (!strcmp((char *)a,(char *)b));
}


